import uno, unohelper
from operator import itemgetter

from com.sun.star.frame import (XStatusListener,
    XDispatchProvider,
    XDispatch,
    XControlNotificationListener,
    FeatureStateEvent)
from com.sun.star.lang import (XInitialization,
    XServiceInfo,
    XEventListener)
from com.sun.star.util import XCloseListener


IMPL_NAME = "twohot.frame.Protocol"


# CellFlags
VALUE = 1
STRING = 4
FORMULA = 16


class DocumentCloseListener(unohelper.Base, XCloseListener):
    """ helps to dispose the frame keeper. """
    def __init__(self, parent):
        self.parent = parent
    # XEventListener
    def disposing(self, ev):
        self.parent.dispose()
        self.parent = None

    def queryClosing(self, ev, ownership): pass
    def notifyClosing(self, ev): pass


class Dispatcher(unohelper.Base, XDispatch, XControlNotificationListener):
    def __init__(self, frame, url):
        self.frame = frame
        self.state = False
        self.listeners = []
        self.url = url

    # XDispatch
    def dispatch(self, url, args):
        if url.Path[:13]=='ToggleButton?':
            self.state = not self.state
            self.broadcast_state()
            task = url.Path[13:]
        else:
            task = url.Path
        print (task)

        self.doThis(task)

    def broadcast_state(self):
        ev = self.create_simple_event(self.url, self.state)
        self.status_changed(ev)

    def status_changed(self, ev):
        for listener in self.listeners:
            listener.statusChanged(ev)

    def addStatusListener(self, listener, url):
        try:
            self.listeners.find(listener)
        except:
            self.listeners.append(listener)
            self.broadcast_state() # set current state

    def removeStatusListener(self, listener, url):
        try:
            m = self.listeners.find(listener)
            self.listeners.pop(n)
        except:
            pass

    # XControlNotificationListener
    def controlEvent(self, ev): pass

    def create_simple_event(self, url, state, enabled=True):
        return FeatureStateEvent(self, url, "", enabled, False, state)


    def doThis(self, task):
        pass

class FrameKeeper(object):
    def __init__(self, frame):
        self.frame = frame
        self.dispatchers = {}

        self.add_document_listener()

    def __eq__(self, other):
        try:
            return self == other or self.frame == other
        except:
            return False

    def add_document_listener(self):
        doc = self.frame.getController().getModel()
        doc.addCloseListener(DocumentCloseListener(self))

    def dispose(self):
        self.frame = None
        self.dispatchers.clear()
        _remove_keeper(self)

    def add(self, url, value):
        """ add dispatcher. """
        dispatcher = self.dispatchers.get(url, None)
        if dispatcher is None:
            self.dispatchers[url] = value

    def get(self, url):
        return self.dispatchers.get(url, None)


class FrameContainer(object):
    """ keeps FrameKeepers. """

    def __init__(self):
        self.items = []

    def _find(self, value):
        i = 0
        for item in self.items:
            if item == value :
                return i
            i += 1
        raise KeyError()

    def add(self, keeper):
        try:
            self._find(keeper)
        except:
            self.items.append(keeper)

    def remove(self, keeper):
        try:
            n = self._find(keeper)
            self.items.pop(n)
        except:
            pass

    def get(self, value):
        try:
            n = self._find(value)
            return self.items[n]
        except:
            raise


# keeps all frames as wrapped objects
_frames = FrameContainer()

def _remove_keeper(keeper):
    _frames.remove(keeper)


def _add_dispatcher(frame, url):
    """ find frame keeper or create it if not found. """
    dispatcher = None
    try:
        item = _frames.get(frame)
        dispatcher = item.get(url.Main)
    except:
        item = FrameKeeper(frame)
        _frames.add(item)

    if dispatcher is None:
        dispatcher = Dispatcher(frame, url)
        item.add(url.Main, dispatcher)
    return dispatcher


#def _create_dispatcher(frame, url):
#    dispatcher = None
#    if url.protocol == "twohot.frame.Ph:":
#        dispatcher = Dispatcher(frame, url)
#    return dispatcher

#----------------------------------------------------------------------------

def prepare(oSheets):
    pass


#----------------------------------------------------------------------------

def enum(startCol, endCol):
    pass

#----------------------------------------------------------------------------

def toggleCols(oSheet,cols):
    pass

#-------------------------------------------------------------------------------

class Ph(unohelper.Base, XInitialization, XDispatchProvider, XServiceInfo):
    def __init__(self, ctx, *args):
        self.frame = None

    # XInitialization
    def initialize(self, args):
        if len(args) > 0:
            self.frame = args[0]

    # XDispatchProvider
    def queryDispatch(self, url, name, flag):
        dispatch = None
        if url.Protocol == "twohot.frame.Ph:":
            try:
                dispatch = _add_dispatcher(self.frame, url)
            except Exception as e:
                print(e)
        return dispatch

    def queryDispatches(self, descs):
        pass

    # XServiceInfo
    def supportsService(self, name):
        return (name == "com.sun.star.frame.ProtocolHandler")
    def getImplementationName(self):
        return IMPL_NAME
    def getSupportedServiceNames(self):
        return ("com.sun.star.frame.ProtocolHandler",)

g_ImplementationHelper = unohelper.ImplementationHelper()
g_ImplementationHelper.addImplementation(
    Ph, IMPL_NAME, ("com.sun.star.frame.ProtocolHandler",),)
